import java.io.*;
import java.util.*;

import javax.vecmath.*;

public class BeaconLocater
{
    private HashMap ugFilenamesToSpaceNames;
    private HashMap spaceNamesToBeacons;
    private HashMap beaconsToPositions;

    // TODO: better way of determning size of hash table
    public BeaconLocater()
    {
	ugFilenamesToSpaceNames = new HashMap(11);
	spaceNamesToBeacons = new HashMap(53);
	beaconsToPositions = new HashMap(101);
    }

    // adds a beacon to the space with coordinates position
    // if it already exists, will overwrite
    public void addBeacon(String space, String beaconName,
			  String beaconId, Vector3d position)
    {
	String beaconKey = beaconName + " " + beaconId;
	spaceNamesToBeacons.put(space, beaconKey);
	beaconsToPositions.put(beaconKey, position);
    }

    /* TODO: support this later, probably
       public void removeBeacon(String beaconName, String beaconId)
       {
       String beaconKey = beaconName + " " + beaconId;
       spaceNamesToBeacons.remove(
       beaconsToPositions.remove(beaconKey);
       }
    */

    // TODO: this isn't safe...but is more efficient
    public HashMap getBeaconLocationMap()
    {
	return beaconsToPositions;
    }

    // TODO: think of a way to figure out the size of the hashtables
    public static BeaconLocater loadFile(String filename)
	throws IOException
    {
	StringTokenizer strtok;

	BeaconLocater bl = new BeaconLocater();
	int lineNum = 1;
	String command;
	String ugFilename = null;
	String space = null;

	String beaconName;
	String beaconId;
	String beaconKey; // = beaconname + " " + id
	float x, y, z;

	FileReader fr = new FileReader(filename);
	BufferedReader br = new BufferedReader(fr);
	String line = br.readLine();
	while (line != null)
	    {
		if (line.charAt(0) != '#')
		    try
			{
			    strtok = new StringTokenizer(line);
			    command = strtok.nextToken();
			    if (command.equals("UGFILE"))
				if (ugFilename != null)
				    error("Badly formed file on line: "
					  + lineNum + ":" + line
					  + "must declare ENDUGFILE");
				else
				    {
					ugFilename = strtok.nextToken();
					if (!(bl.ugFilenamesToSpaceNames.containsKey(ugFilename)))
					    bl.ugFilenamesToSpaceNames.put
						(ugFilename, new LinkedList());
				    }
			    // TODO: you have to check the ug file
			    // to see if the space exists - so parse
			    // dynamically!
			    else if (command.equals("SPACE"))
				if (ugFilename == null)
				    error("Badly formed file on line: "
					  + lineNum + ":" + line
					  + " must be in UGFILE section");
				else if (space != null)
				    error("Badly formed file on line: "
					  + lineNum + ":" + line
					  + " must declare ENDSPACE");
				else
				    space = strtok.nextToken(); // should look in ugfile here..			   
			    // else
			    // TODO: put into list!
			    else if (command.equals("ENDSPACE"))
				space = null;
			    else if (command.equals("ENDUGFILE"))
				ugFilename = null;		
			    else if (command.equals("BEACON"))
				{
				    if (space == null)
					error("Badly formed file on line: "
					      + lineNum + ":" + line
					      + " SPACE must be declared");
				    else
					{
					    beaconName = strtok.nextToken();
					    beaconId = strtok.nextToken();
					    beaconKey = beaconName + " "
						+ beaconId;
					    x = Float.parseFloat
						(strtok.nextToken());
					    y = Float.parseFloat
						(strtok.nextToken());
					    z = Float.parseFloat
						(strtok.nextToken());
					    bl.beaconsToPositions.put
						(beaconKey, new Vector3d
						 (x,y,z));
					}
				}
			}
		    catch (NoSuchElementException nee)
			{
			    System.out.println("Badly formed line: " +
					       lineNum);
			}
		++lineNum;
		line = br.readLine();
	    }
	return bl;
    }

    private static void error(String message)
    {
	throw new RuntimeException(message);
    }

    public static void main(String[] args)
    {
	try
	    {
		BeaconLocater.loadFile(args[0]);
	    }
	catch (Exception e) {}
    }
}
