import java.io.*;
import java.util.*;

import javax.vecmath.*;
import Jama.*;

// r = sigma(r(i) * exp( - (currentTime - t(i)) / timeConstant )
// TODO: provide ability to remove beacon names
// Jason's API must provide the correlation between
// spaces and the beacons in them / filenames
// then we remove once the space has changed
public class BeaconProcessor
    implements Callback
{
    private PointerAppGUI pag;
    private UniGrafixParser ugp;

    // TODO: different algorithms, use static ints
    // TODO: 10 second updater instead of every update?
    private int FILTERING_ALGORITHM = 0; // exponential weighting
    
    private long FILTER_CUTOFF = 10000;
    private double timeConstant = 10000;

    private HashMap beaconLocations; // String => Vector3d
    // keyString = beaconName + " " + beaconId
    private HashMap beaconNamesToDistanceListsListener1;
    private HashMap beaconNamesToDistanceListsListener2;
    private HashMap beaconNamesToDistanceEstimatesListener1;
    private HashMap beaconNamesToDistanceEstimatesListener2;

    private Vector3f listener1Position;
    private Vector3f listener2Position;
    private float laserDistance;

    public BeaconProcessor(PointerAppGUI iPAG, UniGrafixParser iUGP)
    {
        pag = iPAG;
        ugp = iUGP;
	beaconLocations = new HashMap(17);
	beaconNamesToDistanceListsListener1 = new HashMap(17);
	beaconNamesToDistanceListsListener2 = new HashMap(17);
	beaconNamesToDistanceEstimatesListener1 = new HashMap(17);
	beaconNamesToDistanceEstimatesListener2 = new HashMap(17);
        listener1Position = new Vector3f();
        listener2Position = new Vector3f();
    }

    // copies over with replacement!
    // requires that beaconMap be a HashMap from Strings (beacon names)
    // to Vector3d's
    public void addBeaconLocations(HashMap beaconMap)
    {
	beaconLocations.putAll(beaconMap);
    }

    public long getFilterCutoff()
    {
	return FILTER_CUTOFF;
    }

    public double getTimeConstant()
    {
	return timeConstant;
    }
    
    public void setFilterCutoff(long t)
    {
	if (t > 0)
	    FILTER_CUTOFF = t;
	else
	    throw new RuntimeException("Filter Cutoff must be > 0");
    }

    public void setTimeConstant(double a)
    {
	timeConstant = a;
    }

    public HashMap getDistanceEstimates(int listenerNum)
    {
	switch(listenerNum)
	    {
	    case 1:
		return beaconNamesToDistanceEstimatesListener1;
	    case 2:
		return beaconNamesToDistanceEstimatesListener2;
	    default:
		throw new IllegalArgumentException("Invalid listener number");
	    }
    }

    public void callback(Object message, String messageType, String id)
    {
        if (messageType.equals("Cricket"))
            {
                int listenerNum = 0;
                if (id.equals("Listener 1"))
                    listenerNum = 1;
                else if (id.equals("Listener 2"))
                    listenerNum = 2;

                update( (CricketData)message, listenerNum );

                BoundingBox3f bbox = ugp.getBoundingBox();
                Vector3f position = solvePosition
                    (listenerNum, 0.001, bbox.minZ, bbox.maxZ);
                if ( position != null &&
                     !(Float.isNaN(position.x)) &&
                     !(Float.isNaN(position.y)) &&
                     !(Float.isNaN(position.z)) )
                    {
                        ArrayList a = new ArrayList(2);
                        a.add(position);
                        a.add( getDistanceEstimates(listenerNum) );
                        pag.callback(a, "repaint", id);
                        if (listenerNum == 1)
                            listener1Position.set(position);
                        else if (listenerNum == 2)
                            listener2Position.set(position);
                    }
            }
        else if (messageType.equals("laser"))
            {
                laserDistance = Float.parseFloat((String)message) * 39.37f;
                pag.callback(message, "laser", id);
            }
    }

    public Vector3f getListener1Position()
    {
	return new Vector3f(listener1Position);
    }

    public Vector3f getListener2Position()
    {
	return new Vector3f(listener2Position);
    }
    
    public float getLaserDistance()
    {
	return laserDistance;
    }

    // returns the position of the point located by the
    // two listeners and laser
    public Vector3f getDotPosition()
    {
        // returnVector = listener1Position
        //    + laserDistance*(listener1Position - listener2Position);
        Vector3f unitBetween = new Vector3f(listener1Position);
        unitBetween.sub(listener2Position);
        unitBetween.normalize();
        unitBetween.scale(laserDistance);
        Vector3f returnVector = new Vector3f(listener1Position);
        returnVector.add(unitBetween);
        return returnVector;
    }

    public synchronized void update(CricketData data, int listenerNum)
    {
	HashMap distanceMap = null;
	HashMap estimateMap = null;

        if ( listenerNum == 1 )
	    {
		distanceMap = beaconNamesToDistanceListsListener1;
		estimateMap = beaconNamesToDistanceEstimatesListener1;
	    }
	else if ( listenerNum == 2 )
	    {
		distanceMap = beaconNamesToDistanceListsListener2;
		estimateMap = beaconNamesToDistanceEstimatesListener2;
	    }

	String key = data.getName() + " " + data.getId();
	// if this particular beacon isn't in the distanceMap
	// i.e. this listener hasn't heard it yet
	LinkedList distanceList;
	if ( !(distanceMap.containsKey(key)) )
	    {
		distanceList = new LinkedList();
		distanceMap.put(key, distanceList);
		estimateMap.put(key, null);
	    }
	else
	    distanceList = (LinkedList)(distanceMap.get(key));
	distanceList.addLast(data);

	switch(FILTERING_ALGORITHM)
	    {
	    case 0:
		exponentialWeighting(distanceMap, estimateMap);
		break;
	    }
    }

    // distancemap is a map of beaconIds (space + " " + id) -> linked lists
    // of CricketData objects
    // estimateMap maps the same beaconIds -> Double (exponentially
    // weighted distance).  estimateMap is modified
    public synchronized void exponentialWeighting
	(HashMap distanceMap, HashMap estimateMap)
    {
	long currentTime = System.currentTimeMillis();
	long deltaT;
	double distance;
	double distanceTimesWeightingFactor;
	double sumDistanceTimesWeightingFactors = 0;
	double weightingFactor;
	double sumWeightingFactors = 0;
	LinkedList distanceList;
	CricketData currentCricketData;
	// iterate through list of keys in the hashmap
	for (Iterator keyItr = distanceMap.keySet().iterator();
	     keyItr.hasNext();)
	    {
		// each key should give a linked list of distances collected 
		Object key = keyItr.next();
		distanceList = (LinkedList)(distanceMap.get(key));
		// iterate through list and filter out the ones older than
		// FILTER_CUTOFF (default 10 seconds) milliseconds
		// if it's not older, then exponentially weight
		for (Iterator listItr = distanceList.iterator();
		     listItr.hasNext();)
		    {
			currentCricketData = (CricketData)(listItr.next());
			deltaT = currentTime-currentCricketData.getTimeStamp();
			if ( deltaT > FILTER_CUTOFF )
			    listItr.remove();
			else
			    {
				weightingFactor
				    = Math.exp(-deltaT / timeConstant);
				sumWeightingFactors += weightingFactor;
				distance = currentCricketData.getDist();
				distanceTimesWeightingFactor
				    = distance * weightingFactor;
				sumDistanceTimesWeightingFactors +=
				    distanceTimesWeightingFactor;
			    }
		    }
		// stick it in the estimates map, just one weighted
		// distance estimate per beacon per listener
		estimateMap.put(key,
				new Double(sumDistanceTimesWeightingFactors
				 / sumWeightingFactors));
		sumDistanceTimesWeightingFactors = 0;
		sumWeightingFactors = 0;
	    }
    }

    // returns null if no file was loaded or not enough beacons exist
    // client should check for this case!
    // epsilon will be used to test for 0, specifically, abs(x) < epsilon == 0
    public Vector3f solvePosition(int listenerNumber, double epsilon,
				  double minZ, double maxZ)
    {
	if (beaconLocations == null)
	    {
		System.out.println("Load a beacon location file");
		return null;
	    }
	HashMap distanceEstimates = null;
	if (listenerNumber == 1)
	    distanceEstimates = beaconNamesToDistanceEstimatesListener1;
	else
	    distanceEstimates = beaconNamesToDistanceEstimatesListener2;
	Set estimateKeys = distanceEstimates.keySet();
	int numBeacons = estimateKeys.size();
	Matrix m = null;
	Matrix d = null;
	if (numBeacons > 2)
	    {
		int numRows = numBeacons - 1;
		// stores the difference of the sphere equations
		m = new Matrix(numRows, 3);
		// right hand side of sphere equations, d[i+1]^2 - d[i]^2
		d = new Matrix(numRows, 1);
	    }
	else
	    {
		System.out.println("Insufficient beacons: " + numBeacons);
		return null;
	    }
	// populate matrix with differences
	double diffX, diffY, diffZ, diffD;
	double sumX, sumY, sumZ, sumD;
	Iterator estimateKeysItr = estimateKeys.iterator();
	String beaconKey = (String)(estimateKeysItr.next());
	Vector3d previousLocation = (Vector3d)(beaconLocations.get(beaconKey));
	Vector3d currentLocation;
	double previousDistance
	    = ((Double)(distanceEstimates.get(beaconKey))).doubleValue();
	double currentDistance;
	int rowNum = 0; // row of the matrix = beacon number - 1
	while(estimateKeysItr.hasNext())
	    {
		beaconKey = (String)(estimateKeysItr.next());
		currentLocation = (Vector3d)(beaconLocations.get(beaconKey));
		currentDistance = ((Double)(distanceEstimates.get
					    (beaconKey))).doubleValue();
		diffX = currentLocation.x - previousLocation.x;
		sumX = currentLocation.x + previousLocation.x;
		diffY = currentLocation.y - previousLocation.y;
		sumY = currentLocation.y + previousLocation.y;
		diffZ = currentLocation.z - previousLocation.z;
		sumZ = currentLocation.z + previousLocation.z;
		// yes, distances are the other way around
		diffD = previousDistance - currentDistance;
		sumD = previousDistance + currentDistance;
		
		m.set(rowNum, 0, 2*diffX);
		m.set(rowNum, 1, 2*diffY);
		m.set(rowNum, 2, 2*diffZ);
		d.set(rowNum, 0, diffD*sumD + diffX*sumX
		      + diffY*sumY + diffZ*sumZ);

		previousLocation = currentLocation;
		previousDistance = currentDistance;
		++rowNum;
	    }

	Matrix positionMatrix = null;
	try
	    { positionMatrix = pseudoInverse(m,epsilon).times(d); }
	catch(ArrayIndexOutOfBoundsException aioobe)
	    {
		System.out.println("Under-determined system");
		return null;
	    }
	double x = positionMatrix.get(0,0);
	double y = positionMatrix.get(1,0);
	// TODO: check the math here, z may not always be 0 if it's degenerate
	double z = positionMatrix.get(2,0);
	if (Math.abs(z) < epsilon)
	    {
		Vector3d currentBeaconCoord;
		Double currentDist;
		double currentD;
		double beaconX, beaconY, beaconZ;
		double rhs, discrim;
		double z1, z2, zInside;
		double zSum = 0;
		int numZValues = 0;
		for(Iterator zItr=estimateKeys.iterator(); zItr.hasNext();)
		    {
			beaconKey = (String)(zItr.next());
			currentBeaconCoord
			    = (Vector3d)(beaconLocations.get(beaconKey));
			currentDist
			    = ((Double)(distanceEstimates.get(beaconKey)));
			currentD = currentDist.doubleValue();
			beaconX = currentBeaconCoord.x;
			beaconY = currentBeaconCoord.y;
			beaconZ = currentBeaconCoord.z;
			diffX = x - beaconX;
			diffY = y - beaconY;
			rhs = currentD*currentD - diffX*diffX
			    - diffY*diffY - beaconZ*beaconZ;
			discrim = Math.sqrt(4*(beaconZ*beaconZ+rhs));

			z1 = beaconZ + discrim / 2;
			z2 = beaconZ - discrim / 2;
			if (z1 < maxZ && z1 > minZ)
			    {
				zInside = z1;
				zSum += zInside;
				++numZValues;
			    }
			else if (z2 < maxZ && z2 > minZ)
			    {
				zInside = z2;
				zSum += zInside;
				++numZValues;
			    }
			// otherwise, it's invalid and we don't increment
			// numZs and just go on
		    }
		if (numZValues > 0)
		    {
			z = zSum / numZValues;
			return new Vector3f((float)x,(float)y,(float)z);
		    }
		else
		    {
			System.out.println
			    ("Insufficient Z values to determine z");
			return null;
		    }
	    }
	else
	    return new Vector3f((float)x,(float)y,(float)z);
    }

    // guaranteed pseudoinverse using SVD technique
    // epsilon will be used to test for 0, specifically, abs(x) < epsilon == 0
    public static Matrix pseudoInverse(Matrix m, double epsilon)
    {
	SingularValueDecomposition svd = m.svd();
	Matrix ut = svd.getU().transpose();
	Matrix st = svd.getS().transpose();
	Matrix v = svd.getV();
	double val;
	for (int i=0; i<st.getRowDimension(); ++i)
	    for (int j=0; j<st.getColumnDimension(); ++j)
		{
		    val = st.get(i,j);
		    if (Math.abs(val) > epsilon)
			st.set(i, j, 1 / val);
		}

	return v.times(st).times(ut);
    }
}
