import java.util.*;
import java.io.*;
import java.net.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BeaconSim implements MouseListener
{
    private JFrame frame;
    private JPanel panel;
    private JLabel addressLabel;
    private JTextField addressTextField;
    private JLabel commandLabel;
    private JTextField commandTextField;
    private JButton connectButton;
    private JButton disconnectButton;
    private JButton sendStringButton;
    private JTextField listenerOneTextField;
    private JButton listenerOneButton;
    private JTextField listenerTwoTextField;
    private JButton listenerTwoButton;
    private JLabel listenerOneCommandLabel;
    private JTextField listenerOneCommandTextField;
    private JLabel listenerTwoCommandLabel;
    private JTextField listenerTwoCommandTextField;

    private boolean clientConnected;
    private Socket clientSocket;
    private InputStream clientSocketStreamIn;
    private OutputStream clientSocketStreamOut;

    private boolean serverOneConnected;
    private boolean serverTwoConnected;
    private ServerSocket serverOneSocket;
    private ServerSocket serverTwoSocket;
    private OutputStream socketOneStreamOut;
    private OutputStream socketTwoStreamOut;


    public BeaconSim()
    {
        System.out.println("constructor");
        initGui();
        initNet();

        show();
    }

    private void initGui()
    {
        frame = new JFrame("Beacon Simulator");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        panel = new JPanel();
        panel.setPreferredSize(new Dimension(500, 500));
        
        addressLabel = new JLabel("address:port");
        addressTextField = new JTextField("limn.lcs.mit.edu:5001");
        commandLabel = new JLabel("Interpreter Command");
        commandTextField = new JTextField(20);
        
        connectButton = new JButton("Connect");
        disconnectButton = new JButton("Disconnect");
        sendStringButton = new JButton("Send string");

        listenerOneTextField = new JTextField("limn.lcs.mit.edu:2947", 20);
        listenerTwoTextField = new JTextField("limn.lcs.mit.edu:2948", 20);

        listenerOneButton = new JButton("Send Listener One = address:port");
        listenerTwoButton = new JButton("Send Listener Two = address:port");
        
        listenerOneCommandLabel = new JLabel("L1 Command");
        listenerOneCommandTextField = new JTextField(20);
        listenerTwoCommandLabel = new JLabel("L2 Command");
        listenerTwoCommandTextField = new JTextField(20);

        connectButton.addMouseListener(this);
        disconnectButton.addMouseListener(this);
        sendStringButton.addMouseListener(this);
        listenerOneButton.addMouseListener(this);
        listenerTwoButton.addMouseListener(this);

        panel.add(addressLabel);
        panel.add(addressTextField);
        panel.add(connectButton);
        panel.add(disconnectButton);
        panel.add(commandLabel);
        panel.add(commandTextField);
        panel.add(sendStringButton);
        panel.add(listenerOneTextField);
        panel.add(listenerOneButton);
        panel.add(listenerTwoTextField);
        panel.add(listenerTwoButton);

        frame.getContentPane().add(panel);
    }

    public void mouseClicked(MouseEvent e)
    {
        Object source = e.getSource();
        if ( source == connectButton )
            {
                if ( !clientConnected )
                    {
                        String addressString = addressTextField.getText();
                        StringTokenizer strtok
                            = new StringTokenizer(addressString, ":");
                        String address = strtok.nextToken();
                        int port = Integer.parseInt(strtok.nextToken());
                        try
                            {
                                clientSocket = new Socket(address, port);
                                clientSocketStreamIn
                                    = clientSocket.getInputStream();
                                clientSocketStreamOut
                                    = clientSocket.getOutputStream();
                                clientConnected = true;
                            }
                        catch(Exception ex) {}                           
                    }
            }
        else if ( source == disconnectButton )
            {
                if ( clientConnected )
                    {
                        try
                            {
                                clientSocketStreamIn.close();
                                clientSocketStreamOut.close();
                                clientConnected = false;
                                clientSocket = null;
                                clientSocketStreamIn = null;
                                clientSocketStreamOut = null;
                            }
                        catch(Exception ex)
                            {
                                System.out.println("Error closing, try again");
                            }                        
                    }
            }
        else if ( source == sendStringButton )
            {
                writeClient(commandTextField.getText());
            }
        else if ( source == listenerOneButton )
            {
                String addressPort
                    = listenerOneTextField.getText();
                StringTokenizer strtok = new StringTokenizer
                    (addressPort, ":");
                String address = strtok.nextToken();
                String port = strtok.nextToken();
                try
                    {
                        startListenerSim(port, 1);
                        writeClient("listener1 " + address + " " + port);
                        System.out.println("Waiting for connection");
                        Socket s = serverOneSocket.accept();
                        socketOneStreamOut = s.getOutputStream();
                        serverOneConnected = true;
                        System.out.println("listener1 connected");
                    }
                catch(IOException ie)
                    {
                        System.out.println
                            ("Failed to create serversocket, try again");
                    }
            }
        else if ( source == listenerTwoButton )
            {
                String addressPort
                    = listenerTwoTextField.getText();
                StringTokenizer strtok = new StringTokenizer
                    (addressPort, ":");
                String address = strtok.nextToken();
                String port = strtok.nextToken();
                try
                    {
                        startListenerSim(port, 2);
                        writeClient("listener2 " + address + " " + port);
                        System.out.println("Waiting for connection");
                        Socket s = serverTwoSocket.accept();
                        socketTwoStreamOut = s.getOutputStream();
                        serverTwoConnected = true;
                        System.out.println("listener2 connected");
                    }
                catch(IOException ie)
                    {
                        System.out.println
                            ("Failed to create serversocket, try again");
                    }
            }
    }

    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}


    private void show()
    {
        frame.pack();
        frame.setVisible(true);
    }

    private void initNet()
    {
        clientConnected = false;
        serverOneConnected = false;
        serverTwoConnected = false;
    }
    
    private void startListenerSim(String port, int num)
        throws IOException
    {
        int p = Integer.parseInt(port);
        if ( num == 1 )
            {
                serverOneSocket = new ServerSocket(p);
            }
        else if ( num == 2 )
            {
                serverTwoSocket = new ServerSocket(p);
            }
    }

    private void writeClient(String s)
    {
        if ( clientConnected )
            {
                try
                    {
                        clientSocketStreamOut.write( (s + "\n").getBytes() );
                    }
                catch(IOException ie)
                    {
                        System.out.println("Error writing, try again");
                    }
            }
        else
            System.out.println("Connect first!");
    }

    private void writeServer(String s, int num)
    {
        /*
          private ServerSocket serverOneSocket;
          private ServerSocket serverTwoSocket;
          private OutputStream socketOneStreamOut;
          private OutputStream socketTwoStreamOut;
        */

        OutputStream stream = null;
        boolean connected = false;

        if (num == 1)
            {
                stream = socketOneStreamOut;
                connected = serverOneConnected;
            }
        else if ( num == 2 )
            {
                stream = socketTwoStreamOut;
                connected = serverTwoConnected;
            }
        if ( connected )
            {
                try
                    {
                        stream.write( (s + "\n").getBytes() );
                    }
                catch(IOException ie)
                    {
                        System.out.println("Error writing, try again");
                    }
            }
    }

    public static void main(String[] args)
    {
        BeaconSim bs = new BeaconSim();
    }
}
