import java.util.*;
import javax.vecmath.Point3f;

// immutable bounding box class
public class BoundingBox3f
{
    public final float minX;
    public final float minY;
    public final float minZ;

    public final float maxX;
    public final float maxY;
    public final float maxZ;

    public BoundingBox3f(float minX, float minY, float minZ,
			 float maxX, float maxY, float maxZ)
    {
	this.minX = minX;
	this.minY = minY;
	this.minZ = minZ;
	this.maxX = maxX;
	this.maxY = maxY;
	this.maxZ = maxZ;
    }

    public boolean contains(Point3f p)
    {
	return ( (minX < p.x) && (maxX > p.x) &&
		 (minY < p.y) && (maxY > p.y) &&
		 (minZ < p.z) && (maxZ > p.z) );
    }

    public String toString()
    {
        return "minX: " + minX + "\tminY: " + minY + "\tminZ: " + minZ +
            "\nmaxX: " + maxX + "\tmaxY: " + maxY + "\tmaxZ: " + maxZ;
    }
    
    // utility methods

    // collection must be of Triangle3f's
    public static BoundingBox3f calculateFromTriangles(Collection triangles)
    {
        float minX = Float.POSITIVE_INFINITY;
        float minY = Float.POSITIVE_INFINITY;
        float minZ = Float.POSITIVE_INFINITY;
        float maxX = Float.NEGATIVE_INFINITY;
        float maxY = Float.NEGATIVE_INFINITY;
        float maxZ = Float.NEGATIVE_INFINITY;
        Triangle3f t;
        for (Iterator itr = triangles.iterator(); itr.hasNext();)
	    {
		t = (Triangle3f)(itr.next());
		if (minX > t.v1.x)
		    minX = t.v1.x;
		if (minY > t.v1.y)
		    minY = t.v1.y;
		if (minZ > t.v1.z)
		    minZ = t.v1.z;
		if (minX > t.v2.x)
		    minX = t.v2.x;
		if (minY > t.v2.y)
		    minY = t.v2.y;
		if (minZ > t.v2.z)
		    minZ = t.v2.z;
		if (minX > t.v3.x)
		    minX = t.v3.x;
		if (minY > t.v3.y)
		    minY = t.v3.y;
		if (minZ > t.v3.z)
		    minZ = t.v3.z;
		if (maxX < t.v1.x)
		    maxX = t.v1.x;
		if (maxY < t.v1.y)
		    maxY = t.v1.y;
		if (maxZ < t.v1.z)
		    maxZ = t.v1.z;
		if (maxX < t.v2.x)
		    maxX = t.v2.x;
		if (maxY < t.v2.y)
		    maxY = t.v2.y;
		if (maxZ < t.v2.z)
		    maxZ = t.v2.z;
		if (maxX < t.v3.x)
		    maxX = t.v3.x;
		if (maxY < t.v3.y)
		    maxY = t.v3.y;
		if (maxZ < t.v3.z)
		    maxZ = t.v3.z;
	    }
        return new BoundingBox3f(minX, minY, minZ, maxX, maxY, maxZ);
    }

    // Collection must be of BoundingBox3f's
    public static BoundingBox3f calculateFromBoundingBoxes(Collection c)
    {
        float minX = Float.POSITIVE_INFINITY;
        float minY = Float.POSITIVE_INFINITY;
        float minZ = Float.POSITIVE_INFINITY;
        float maxX = Float.NEGATIVE_INFINITY;
        float maxY = Float.NEGATIVE_INFINITY;
        float maxZ = Float.NEGATIVE_INFINITY;
        BoundingBox3f currentBoundingBox;
        for (Iterator itr = c.iterator(); itr.hasNext();)
            {
                currentBoundingBox = (BoundingBox3f)(itr.next());
                if (currentBoundingBox.minX < minX)
                    minX = currentBoundingBox.minX;
                if (currentBoundingBox.minY < minY)
                    minY = currentBoundingBox.minY;
                if (currentBoundingBox.minZ < minZ)
                    minZ = currentBoundingBox.minZ;
                if (currentBoundingBox.maxX > maxX)
                    maxX = currentBoundingBox.maxX;
                if (currentBoundingBox.maxY > maxY)
                    maxY = currentBoundingBox.maxY;
                if (currentBoundingBox.maxZ > maxZ)
                    maxZ = currentBoundingBox.maxZ;
            }
        return new BoundingBox3f(minX, minY, minZ, maxX, maxY, maxZ);
    }
}
