import java.util.*;
import java.io.*;
import java.net.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Cricket3DGUI
{
    private JFrame frame;
    private Container contentPane;
    private JPanel mainPanel;

    private JLabel portLabel;
    private JTextField portTextField;
    private JLabel blFilenameLabel;
    private JTextField blFilenameTextField;
    private JButton startButton;
    private JTextField statusTextField;

    // main listening thread for initial connections
    private boolean serverStarted;
    private Cricket3DServer cricket3DServer;

    // separate clients for the positions and distance
    private CricketClient cricketClient1;
    private CricketClient cricketClient2;
    private LaserClient laserClient;

    // linear algebra solver for position
    private UniGrafixParser ugp;
    private BeaconLocater beaconLocater; // parses the bl file
    private BeaconProcessor beaconProcessor;

    // Java3D GUI
    PointerAppGUI pag;

    // Database
    CricketDatabase cdb;

    public Cricket3DGUI()
    {
        initVars();
        initGUI();
        show();
    }
    
    private void initVars()
    {
        serverStarted = false;
        cricket3DServer = new Cricket3DServer(this);
    }
    
    private void initGUI()
    {
        frame = new JFrame("Cricket 3D");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        contentPane = frame.getContentPane();
        mainPanel = new JPanel();
        mainPanel.setPreferredSize(new Dimension(500,500));

        portLabel = new JLabel("Port");
        portTextField = new JTextField("5001");
        blFilenameLabel = new JLabel("BL Filename");
        blFilenameTextField = new JTextField("NE43.bl");
        // TODO: file chooser
        statusTextField = new JTextField("Status", 20);
        
        startButton = new JButton("Start/Stop Server");
        startButton.addActionListener
            (new ActionListener()
                {
                    public void actionPerformed(ActionEvent e)
                    {
                        if ( !serverStarted )
                            {
                                // TODO: check port/filename
                                int port = Integer.parseInt
                                    (portTextField.getText());
                                String filename =
                                    blFilenameTextField.getText();
                                cricket3DServer.start(port, filename);
                                serverStarted = true;
                            }
                        else
                            {
                                cricket3DServer.stop();
                                serverStarted = false;
                            }
                    }
                }
             );
        
        mainPanel.add(portLabel);
        mainPanel.add(portTextField);
        mainPanel.add(blFilenameLabel);
        mainPanel.add(blFilenameTextField);
        mainPanel.add(startButton);
        mainPanel.add(statusTextField);

        contentPane.add(mainPanel);
    }

    private void show()
    {
        frame.pack();
        frame.setVisible(true);
    }

    public void writeGUI(String s)
    {
        statusTextField.setText(s);
        System.out.println(s);
    }

    public void disableServer()
    {
        startButton.setEnabled(false);
    }

    public void setCricketClient1(CricketClient c)
    {
        cricketClient1 = c;
    }

    public void setCricketClient2(CricketClient c)
    {
        cricketClient2 = c;
    }

    public void setLaserClient(LaserClient l)
    {
        laserClient = l;
    }

    public void startVisualization()
    {
        /*
	  if ( cricketClient1 != null &&
	  cricketClient2 != null &&
	  laserClient != null )
	  {
	  }
        */
        try
            {
                String blFilename = blFilenameTextField.getText();
                System.out.println("Loading: " + blFilename + "...");
                beaconLocater = BeaconLocater.loadFile(blFilename);
                // TODO: use the ugFilenamesToSpaceNames thing in beaconLocator
                // to switch between spaces
                ugp = new UniGrafixParser("NE43.2.3D.ug");
                pag = new PointerAppGUI
                    (ugp, beaconLocater.getBeaconLocationMap());
                beaconProcessor = new BeaconProcessor(pag, ugp);
                beaconProcessor.addBeaconLocations
                    (beaconLocater.getBeaconLocationMap());

                // TODO: of course, make it parse for the filename	     
                // database doesn't work right now anyway
		cdb = new CricketDatabase("NE43.2.cdb", beaconProcessor);

		if (cricketClient1 != null)
		    cricketClient1.register(beaconProcessor);
		if (cricketClient2 != null)
		    cricketClient2.register(beaconProcessor);
		if (laserClient != null)
		    laserClient.register(beaconProcessor);
		if (cricketClient1 != null)
		    cricketClient1.start();
		if (cricketClient2 != null)
		    cricketClient2.start();
		if (laserClient != null)
		    laserClient.start();

		// TODO: this is kinda weird...
		FlashlightRepainter flashlightRepainter = 
		    new FlashlightRepainter(beaconProcessor);
		javax.swing.Timer repaintTimer =
		    new javax.swing.Timer(3000, flashlightRepainter);

		repaintTimer.start();
                // TODO: dispose all?
                // frame.dispose();
            }
        catch(IOException ie) {}
    }

    public void addPoint(String label)
    {
        cdb.addPoint(label);
    }

    public String sphereQuery(float radius)
    {
        return cdb.sphereQuery(radius);
    }

    public static void main(String[] args)
    {
        Cricket3DGUI c = new Cricket3DGUI();
    }
}
