import java.io.*;
import java.net.*;

public class Cricket3DServer implements Runnable
{
    private Cricket3DGUI gui;

    private boolean started;
    private boolean stopped;
    private int port;
    private String blFilename;

    private ServerSocket serverSocket;
    private Socket[] clientSockets;
    private int numConnections;

    public Cricket3DServer(Cricket3DGUI iGUI)
    {
        gui = iGUI;
        resetVars();
    }

    private void resetVars()
    {
        started = false;
        stopped = true;
        port = -1;
        blFilename = null;
        serverSocket = null;
        clientSockets = new Socket[2];
        numConnections = 0;
    }

    public void start(int iPort, String iBLFilename)
    {
        if ( started )
            return;
        else
            {
                gui.writeGUI("Starting Server...");
                started = true;
                stopped = false;
                port = iPort;
                blFilename = iBLFilename;
                (new Thread(this)).start();
                gui.writeGUI("Started");
            }
    }

    public void stop()
    {
        gui.writeGUI("Stopping Server...");
        started = false;
        try
            {
                if (numConnections < 2)
                    serverSocket.close();
            }
        catch(IOException ie)
            {
                gui.writeGUI("Error stopping server, try again");
                return;
            }
    }

    public boolean stopped()
    {
        return stopped;
    }

    public void run()
    {
        try
            {
                initServer();
            }
        catch(IOException ie)
            {
                resetVars();
                gui.writeGUI("Error starting server, server stopped");
                started = false;
                stopped = true;
                return;
            }
        while ( started && (numConnections < 2) )
            {
                try
                    {
                        listen();
                        gui.writeGUI
                            ("Client " + numConnections + " has connected");
                    }
                catch(SocketException se)
                    {
                        // catch for closing server
                    }
                catch(IOException ie)
                    {
                        resetVars();
                        gui.writeGUI("Error accepting connection");
                        ie.printStackTrace();
                        return;
                    }
            }
        started = false;
        stopped = true;
        gui.writeGUI("Stopped");
        if ( numConnections == 2 )
            {
                gui.disableServer();
                gui.writeGUI("Starting interpreters");
            }
    }
    
    private void initServer()
        throws IOException
    {
        serverSocket = new ServerSocket(port, 2);
    }

    private void listen()
        throws IOException
    {       
        clientSockets[numConnections] = serverSocket.accept();
        (new Thread(new Cricket3DInterpreter
                    (clientSockets[numConnections], gui))).start();
        ++numConnections;
    }
}
