import java.util.*;
import java.io.*;
import java.net.*;

// $Cricket2,space=NTT1,id=20,dist=102
public class CricketClient extends Thread
{
    // Network I/O
    private String ip;
    private int port;
    private Socket socket;
    private BufferedReader socketReader;
    private OutputStream outputStream;

    // Status
    private String id;
    private boolean online;

    // Callback list
    private LinkedList callbacks;

    public CricketClient(String ip, int port, String id)
    {
	this.ip = ip;
	this.port = port;
	this.id = id;
	online = false; // only there so that connect() can be reused
	callbacks = new LinkedList();
    }

    public synchronized void connect()
	throws IOException
    {
	if (online)
	    disconnect();

	socket = new Socket(ip, port);
	socketReader = new BufferedReader
	    (new InputStreamReader(socket.getInputStream()));
	outputStream = socket.getOutputStream();
	online = true;
    }

    public synchronized void connect(String ip, int port)
	throws IOException
    {
	this.ip = ip;
	this.port = port;
	connect();
    }

    public synchronized void disconnect()
	throws IOException
    {
	if (online)
	    {
		outputStream.write("r\n".getBytes());
		socket.close();
		socketReader = null;
		outputStream = null;
		online = false;
	    }
    }

    public synchronized void register(Callback cb)
    {
	callbacks.addLast(cb);
    }

    public synchronized void deregister(Callback cb)
    {
	callbacks.remove(cb);
    }

    public void run()
    {
	// send "r\n" to gpsd, to register
	try
	    { outputStream.write("r\n".getBytes()); }
	catch (IOException ie)
	    {
		System.out.println("Unable to write to socket, stopping...");
		try
		    { disconnect(); }
		catch (IOException iee)
		    { System.out.println("Could not close socket, sorry!"); }
	    }

	String line;
	synchronized(this)
	    {
		while (online)
		    {
			try
			    {
				line = socketReader.readLine();
				update(convertRawToCricketData(line));
			    }
			catch (IOException ie)
			    {
				System.out.println
				    ("Communications error, stopping...");
				try
				    { disconnect(); }
				catch (IOException iee)
				    {
					System.out.println
					    ("Could not close socket, sorry!");
				    }
			    }
		    }
	    }
    }

    private synchronized void update(CricketData cd)
    {
	if (cd != null)
	    for(Iterator itr=callbacks.iterator(); itr.hasNext();)
		((Callback)(itr.next())).callback(cd, "Cricket", id);
    }

    /* Line format for a cricket string as of this rev is:
       $Cricket2,space=SPACENAME,id=IDINTEGER,dist=DISTINTEGER
    */
    public static CricketData convertRawToCricketData(String rawData)
    {
	StringTokenizer st = new StringTokenizer(rawData, ",=");
	String useless = st.nextToken(); // gives "$Cricket2"
	if (useless.equals("GPSD")) // first line is GPSD, ignore it
	    return null;
	else
	    {
		st.nextToken(); // gives "space="
		String name = st.nextToken();
		st.nextToken(); // gives "id="
		// give it the actual id
		int id = Integer.parseInt(st.nextToken(), 16) - 32;
		st.nextToken(); // gives "dist="
		String dist = st.nextToken();
		int parsedDist = Integer.parseInt(dist, 16);
		return new CricketData(name, id,
				       ((double)(parsedDist - 36))
				       * 2.2 / 2.54,  // convert to inches
				       System.currentTimeMillis());
	    }
    }

    public static void main(String[] args)
	throws IOException
    {
	CricketClient cc = new CricketClient("18.24.6.167", 2947, "Listener");
	cc.start();
	while(true);
    }
}
