import java.util.*;
import java.io.*;

import javax.vecmath.*;

public class CricketDatabase
{
    private String filename;
    private BeaconProcessor beaconProcessor;
    private HashMap positionToLabel;
    private HashMap labelToPosition;

    public CricketDatabase
        (String iFilename, BeaconProcessor iBeaconProcessor)
        throws IOException
    {
        filename = iFilename;
        beaconProcessor = iBeaconProcessor;
        positionToLabel = new HashMap(101);
        labelToPosition = new HashMap(101);

        BufferedReader reader
            = new BufferedReader(new FileReader(filename));
        String line = reader.readLine();
        int numLines = Integer.parseInt(line);
        for (int i=0; i<numLines; ++i)
            {
                parseLine(reader.readLine());
            }
        reader.close();
    }

    private void parseLine(String line)
    {
        StringTokenizer strtok = new StringTokenizer(line, " ");
        String first = strtok.nextToken();
        if ( !first.equals("#") )
            {
                float x = Float.parseFloat(strtok.nextToken());
                float y = Float.parseFloat(strtok.nextToken());
                float z = Float.parseFloat(strtok.nextToken());
                Vector3f r = new Vector3f(x,y,z);
                String label = strtok.nextToken();
                positionToLabel.put(r, label);
                labelToPosition.put(label, r);
            }
    }

    public void save()
        throws IOException
    {
        BufferedWriter writer
            = new BufferedWriter(new FileWriter(filename));
        Vector3f position;
        String label;
        for (Iterator itr = positionToLabel.keySet().iterator();
             itr.hasNext();)
            {
                position = (Vector3f)(itr.next());
                label = (String)(positionToLabel.get(position));              
                writer.write
                    ("" + position.x + " " + position.y + " "
                     + position.z + " " + label);
            }
        writer.close();
    }

    public void addPoint(String label)
    {
        Vector3f dotPosition = beaconProcessor.getDotPosition();
        if (label.equals("xyz"))
            label = "(" + dotPosition.x + "," +
                dotPosition.y + "," + dotPosition.z + ")";
        positionToLabel.put(dotPosition, label);
        labelToPosition.put(label, dotPosition);
    }

    public String sphereQuery(float radius)
    {
        float rSquared = radius*radius;
        Vector3f currentPosition;
        Vector3f dotPosition = beaconProcessor.getDotPosition();
        Vector3f temp = new Vector3f();
        String returnStr = "";
        for(Iterator itr=positionToLabel.keySet().iterator(); itr.hasNext();)
            {
                currentPosition = (Vector3f)(itr.next());
                temp.set(currentPosition);
                temp.sub(dotPosition);
                if ( temp.lengthSquared() < rSquared )
                    returnStr = returnStr + ", " +
                        (String)(positionToLabel.get(currentPosition));
            }
        return returnStr;
    }
}
