import java.util.*;
import java.io.*;
import java.net.*;

public class LaserClient extends Thread
{
    // Network I/O
    private String ip;
    private int port;
    private Socket socket;
    private BufferedReader socketReader;
    private OutputStream outputStream;

    // Status
    private String id;
    private boolean online;

    // Callback list
    private LinkedList callbacks;

    public LaserClient(String ip, int port, String id)
    {
	this.ip = ip;
	this.port = port;
	this.id = id;
	online = false; // only there so that connect() can be reused
	callbacks = new LinkedList();
    }

    public synchronized void register(Callback cb)
    {
	callbacks.addLast(cb);
    }

    public synchronized void deregister(Callback cb)
    {
	callbacks.remove(cb);
    }
    public synchronized void connect()
	throws IOException
    {
	if (online)
	    disconnect();

	socket = new Socket(ip, port);
	socketReader = new BufferedReader
	    (new InputStreamReader(socket.getInputStream()));
	outputStream = socket.getOutputStream();
	online = true;
    }

    public synchronized void connect(String ip, int port)
	throws IOException
    {
	this.ip = ip;
	this.port = port;
	connect();
    }
    
    // TODO: send to server disconnect signal
    public synchronized void disconnect()
	throws IOException
    {
	outputStream.write("04stop".getBytes());
	socket.close();
	online = false;
    }

    // TODO: gotta catch the runtimeexception
    public void run()
    {
	try
	    { outputStream.write("05start".getBytes()); }
	catch (IOException ie)
	    { throw new RuntimeException("Communications Error"); }
	
	online = true;
	synchronized(this)
	    {
		try
		    {
			while (online)
			    {
				update(socketReader.readLine());
			    }
		    }
		catch (IOException ie)
		    {
			throw new RuntimeException("Communiations Error");
		    }
	    }
    }

    private synchronized void update(String distance)
    {
	for (Iterator itr=callbacks.iterator(); itr.hasNext();)
	    ((Callback)(itr.next())).callback(distance, "laser", id);
    }

    public static void main(String[] args)
	throws IOException
    {
	LaserClient lc = new LaserClient("frame", 5003, "Laser");
	lc.start();
	while(true);
    }
}
