// Java3D
import javax.media.j3d.*;
import javax.vecmath.*;

// Java3D Utility Classes
import com.sun.j3d.utils.geometry.*;

public class Plane3D extends Shape3D
{
    private Point3f origin;
    private Vector3f uDirection, vDirection;
    private int uSegments, vSegments;
    private Vector3f normal;
    private Color3f color;
    
    public Plane3D(Point3f origin,
		   Vector3f uDirection, Vector3f vDirection,
		   int uSegments, int vSegments,
		   Color3f color)
    {
	super();
	this.origin = origin;
	this.uDirection = uDirection;
	this.vDirection = vDirection;
	this.uSegments = uSegments;
	this.vSegments = vSegments;
	this.color = color;

	Appearance appr = new Appearance();
	appr.setColoringAttributes
	    (new ColoringAttributes(color, ColoringAttributes.SHADE_FLAT));

	// TODO: probably get rid of antialiasing
	LineAttributes lineAttr = new LineAttributes();
	lineAttr.setLineAntialiasingEnable(true);
	appr.setLineAttributes(lineAttr);

	setGeometry(generateGeometry());
	setAppearance(appr);
    }
    
    public Geometry generateGeometry()
    {
	LineArray la = new LineArray( 2*uSegments + 2*vSegments + 4,
				      LineArray.COORDINATES );

	Point3f uMax = new Point3f(origin);
	Point3f vMax = new Point3f(origin);
	uMax.add(uDirection);
	vMax.add(vDirection);
	Point3f interpolatePoint1 = new Point3f();
	Point3f interpolatePoint2 = new Point3f();
	int segmentCount = 0;
	// actually get uSegments+1 with the <= so it actually draws the boundary line

	for (int i=0; i<=uSegments; ++i)
	    {
		interpolatePoint1.interpolate(origin, uMax, ((float)i) / uSegments);
		interpolatePoint2.set(interpolatePoint1);
		interpolatePoint2.add(vDirection);

		la.setCoordinate(segmentCount, interpolatePoint1);
		la.setCoordinate(segmentCount+1, interpolatePoint2);
		segmentCount += 2;
	    }

	for (int j=0; j<=vSegments; ++j)
	    {
		interpolatePoint1.interpolate(origin, vMax, ((float)j) / vSegments);
		interpolatePoint2.set(interpolatePoint1);
		interpolatePoint2.add(uDirection);
		
		la.setCoordinate(segmentCount, interpolatePoint1);
		la.setCoordinate(segmentCount+1, interpolatePoint2);
		segmentCount += 2;
	    }

	return la;
    }
    
}
