import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import javax.vecmath.*;

public class PointerAppGUI
    implements ActionListener, Callback
{
    // Swing Elements
    private JFrame frame;
    private JPanel cricketPanel;
    private JPanel displayPanel;

    // Cricket Panel
    private JButton renderButton;
    private JCheckBox drawAxesCheckBox;
    private JCheckBox drawGridCheckBox;
    private JCheckBox drawBeaconsCheckBox;
    private JCheckBox drawListenerOneRadiiCheckBox;
    private JCheckBox drawListenerTwoRadiiCheckBox;

    // Internal Representation
    private String blFilename;
    private String ugFilename;
    private UniGrafixParser ugp;
    private UniGrafixRenderer ugr;
    
    // TODO: this takes some work...
    // make an addfile/switchlocation command and dynamically adjust
    // the beacon processor's stuff
    // HACK: need reverse map of beacons to ug files
    public PointerAppGUI(UniGrafixParser ugp, HashMap beaconLocationMap)
    {
	createWindows(); // create GUI windows
	setupCricketPanel();
	setupDisplayPanel();
	finish(); // data gathered, display
	// TODO: adjust so that when
	// you move to another "file", re-render with new beaconLocations
	ugr.render(ugp, beaconLocationMap);
    }

    private void createWindows()
    {
	frame = new JFrame("Cricket 3D Pointer");
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	cricketPanel = new JPanel(new GridLayout(10,1));
	displayPanel = new JPanel(new BorderLayout());
	cricketPanel.setPreferredSize(new Dimension(300, 600));
	displayPanel.setPreferredSize(new Dimension(900, 600));

	Container contentPane = frame.getContentPane();
	contentPane.add(cricketPanel, BorderLayout.WEST);
	contentPane.add(displayPanel, BorderLayout.EAST);
    }

    // TODO: need one that loads additional files, maybe graphical
    // server window
    private void setupCricketPanel()
    {
	renderButton = new JButton("Render UniGrafix File");
	renderButton.addActionListener(this);

	drawGridCheckBox = new JCheckBox("Draw Grid", true);
	drawGridCheckBox.addActionListener(this);

	drawAxesCheckBox = new JCheckBox("Draw Axes", true);
	drawAxesCheckBox.addActionListener(this);

	drawBeaconsCheckBox = new JCheckBox("Draw Beacons", true);
	drawBeaconsCheckBox.addActionListener(this);

	drawListenerOneRadiiCheckBox
	    = new JCheckBox("Draw Listener One Radii", false);
	drawListenerOneRadiiCheckBox.addActionListener(this);

	drawListenerTwoRadiiCheckBox
	    = new JCheckBox("Draw Listener Two Radii", false);
	drawListenerTwoRadiiCheckBox.addActionListener(this);

	cricketPanel.add(drawGridCheckBox);
	cricketPanel.add(drawAxesCheckBox);
	cricketPanel.add(drawBeaconsCheckBox);
	cricketPanel.add(drawListenerOneRadiiCheckBox);
	cricketPanel.add(drawListenerTwoRadiiCheckBox);
    }

    private void setupDisplayPanel()
    {
	ugr = new UniGrafixRenderer();
	displayPanel.add(ugr.getCanvas3D());
    }

    private void finish()
    {
	frame.pack();
	frame.setVisible(true);
    }
    
    // =========================== Button Handlers =========================

    public void actionPerformed(ActionEvent e)
    {
	Object source = e.getSource();
	if (source == drawAxesCheckBox)
	    handleDrawAxesCheckBox();
	else if (source == drawGridCheckBox)
	    handleDrawGridCheckBox();
	else if (source == drawBeaconsCheckBox)
	    handleDrawBeaconsCheckBox();
	else if (source == drawListenerOneRadiiCheckBox)
	    handleDrawListenerOneRadiiCheckBox();
	else if (source == drawListenerTwoRadiiCheckBox)
	    handleDrawListenerTwoRadiiCheckBox();
    }
    
    private void handleDrawAxesCheckBox()
    {
	ugr.setAxesState(drawAxesCheckBox.isSelected());
	if (ugr.isRendered())
	    ugr.drawAxes();
    }

    private void handleDrawGridCheckBox()
    {
	ugr.setGridState(drawGridCheckBox.isSelected());
	if (ugr.isRendered())
	    ugr.drawGrid();
    }

    private void handleDrawBeaconsCheckBox()
    {
	ugr.setBeaconsState(drawBeaconsCheckBox.isSelected());
	if (ugr.isRendered())
	    ugr.drawBeacons();
    }

    private void handleDrawListenerOneRadiiCheckBox()
    {
	ugr.setListenerOneRadiiState
	    (drawListenerOneRadiiCheckBox.isSelected());
	if (ugr.isRendered())
	    ugr.drawListenerOneRadii();
    }

    private void handleDrawListenerTwoRadiiCheckBox()
    {
	ugr.setListenerTwoRadiiState
	    (drawListenerTwoRadiiCheckBox.isSelected());
	if (ugr.isRendered())
	    ugr.drawListenerTwoRadii();
    }

    private void reportError(String error)
    {
	JOptionPane.showMessageDialog(frame, error, "Error",
				      JOptionPane.ERROR_MESSAGE);
    }

    // ============================ Callback ================================
    public void callback(Object message, String messageType, String id)
    {
	if (ugr == null)
	    {
		System.out.println("id = " + id + " message = " + message); 
		return;
	    }
        else if (messageType.equals("repaint"))
            {
                int listenerNum = 0;
                if (id.equals("Listener 1"))
		    listenerNum = 1;
		else if (id.equals("Listener 2"))
		    listenerNum = 2;
                ArrayList arrayMessage = (ArrayList)message;
                Vector3f position = (Vector3f)(arrayMessage.get(0));
                HashMap distanceEstimates =
                    (HashMap)(arrayMessage.get(1));
                ugr.updatePosition(listenerNum, position);
                ugr.updateBeaconRadii(listenerNum, distanceEstimates);
                ugr.updateLever();
            }
	else if (messageType.equals("laser"))
	    {
		ugr.updateLaser((String)message, id);
	    }
    }
}
