#include <GL/glut.h>
#include <stdio.h>
#include <math.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

// useful only for a perfect frustum projector!
// float fov = 17.76f;
// float aspect = 1.333f; // = viewportX / viewportY

/* meters
float zNear = 2.0f;
float zFar = 10.0f;
float left = -0.5f;
float right = 0.5f;
float bottom = 0.0f;
float top = 0.75f;
*/

// inches
//float zNear = 78.74f;
float zNear = 78.74f;
float zFar = 393.7f;
float left = -19.685f;
float right = 19.685f;
float bottom = 0.0f;
float top = 29.5275f;

float eyeX = 460.315f;
float eyeY = 593.26f;
float eyeZ = 76.63f;

float targetX = 460.315f;
float targetY = 672.0f;
float targetZ = 76.63f;

int CALIBRATION_POINTS = 24;
int socket_fd;
FILE* socketFile;

float calibration_square[] =
{ 
	440.63, 672, 116,	480, 672, 76.63,
	440.63, 672, 76.63,	480, 672, 116,
	440.63, 672, 96.315,	480, 672, 96.315,
	460.315, 672, 76.63,	460.315, 672, 116
};

void output(int x, int y, char* str)
{
	int len, i;
	glPushMatrix();
	// glRasterPos2f(x, y);
	glTranslatef(x, y, 0);
	len = (int) strlen(str);
	for (i = 0; i < len; i++)
	{
		//glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
		glutStrokeCharacter(GLUT_STROKE_ROMAN, str[i]);
	}
	glPopMatrix();
}

/*
void renderBitmapString(float x, float y, float z, char* str)
{
	char* c;
	glRasterPos3f(x,y,z);
	for(c=str; *c != '\0'; ++c)
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, *c);
}
*/

void drawCalibrationSquare()
{
	int i;
	glBegin(GL_LINES);
	for (i=0; i<CALIBRATION_POINTS; i += 6)
	{
		glVertex3f(
			calibration_square[i],
			calibration_square[i+1],
			calibration_square[i+2]);
		glVertex3f(
			calibration_square[i+3],
			calibration_square[i+4],
			calibration_square[i+5]);			
	}
	glEnd();
}

void drawCircle()
{
	float t = 0;
	glBegin(GL_LINES);
	for (t = 0; t<6.25; t += 0.03)
	{
		glVertex3f(460.315 + 9.8425*cos(t), 672.116f, 96.315 + 9.8425*sin(t));
		glVertex3f(460.315 + 9.8425*cos(t+0.03), 672.116f, 96.315 + 9.8425*sin(t+0.03));
	}
	glEnd();
}

void display(void)
{
	// glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glClear(GL_COLOR_BUFFER_BIT);

	// ============= projective transformation ==============
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//gluPerspective( fov, aspect, zNear, zFar);
	//glViewport(0,0,640,400);
//	glFrustum(left, right, bottom, top, zNear, zFar);
	glFrustum(left, right, bottom, top, zNear - 1.0f, zFar);

	// ============= model/view transformation ==============
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	printf("eye at %f %f %f\ntarget at %f %f %f\n",
	eyeX, eyeY, eyeZ, targetX, targetY, targetZ);
	gluLookAt(
	eyeX, eyeY, eyeZ,	// eye
	targetX, targetY, targetZ, // target
	0.0f, 0.0f, 1.0f	// up vector = (right) cross (target - eye)
	);

	glColor3f(1.0f,1.0f,1.0f);

	drawCalibrationSquare();
	// drawCircle();

	glutSwapBuffers();
}

void idle(void)
{

/*	glutPostRedisplay(); */
}

void keyboard(unsigned char key, int x, int y)
{
	glutPostRedisplay();
}

/*
void visible(int vis)
{
	if (vis == GLUT_VISIBLE)
		glutIdleFunc(idle);
	else
		glutIdleFunc(NULL);
}
*/

void communicate()
{
  char* line = NULL; /* gets allocated by getline() */
  char buffer[255];
  size_t len = 0;
  ssize_t bytesRead;

  char command;
  float ex, ey, ez;
  float tx, ty, tz;
  float distance;

  // printf("Getting next line...\n");

  /* getline includes the newline char */;
  bytesRead = getline(&line, &len, socketFile);
  sscanf(line, "%f %f %f %f %f %f %f\n", &ex, &ey, &ez, &tx, &ty, &tz,
	&distance);
  eyeX = ex; eyeY = ey; eyeZ = ez;
  targetX = tx; targetY = ty; targetZ = tz;
//  zNear = distance;
  glutPostRedisplay();
	
  if ( line != NULL )
    free(line);
}

int main(int argc, char* argv[])
{
	// network initialization
	struct sockaddr_in localAddr, servAddr;
	struct hostent* h;
	int port;
	int socket_fd;
	int status;

	// glutInit(&argc, argv);
	// glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutCreateWindow("Cricket Software Flashlight");
//	glutInitWindowSize(400,400);
	glutFullScreen();

	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutIdleFunc(communicate);
	// glutVisibilityFunc(visible);

	// network code

	if ( argc < 3 )
	  {
	    printf("usage: %s <server> <port>\n", argv[0]);
	    exit(-1);
	  }

	h = gethostbyname(argv[1]);
	sscanf(argv[2], "%d", &port);

	if ( h == NULL )
	  {
	    printf("Unknown host: %s\n", argv[1]);
	    exit(-1);
	  }

	servAddr.sin_family = h->h_addrtype;
	memcpy((char *) &servAddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
	servAddr.sin_port = htons(port);
  
	socket_fd = socket(AF_INET, SOCK_STREAM, 0);

	if ( socket_fd < 0 )
	  {
	    printf("Cannot open socket\n");
	    exit(-1);
	  }

	status = connect(socket_fd, (struct sockaddr*)(&servAddr), sizeof(servAddr));
	if ( status < 0 )
	  {
	    printf("Could not connect\n");
	    exit(-1);
	  }
	printf("Connected");

	socketFile = fdopen(socket_fd, "r");

	glutMainLoop();
	return 0;
}


  //glEnable(GL_DEPTH_TEST);
  //glEnable(GL_CULL_FACE);
  //glEnable(GL_BLEND);
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glEnable(GL_LINE_SMOOTH);
